package com.smartgwt.sample.server;

import java.io.IOException;  
import java.util.ArrayList;  
import java.util.List;  
import java.util.Properties;
import java.util.Random;  
  
import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;

import javax.naming.Context;
import javax.naming.NamingException;

import javax.servlet.ServletException;  
import javax.servlet.http.HttpServlet;  
import javax.servlet.http.HttpServletRequest;  
import javax.servlet.http.HttpServletResponse;  
  
public class StockQuotesServlet extends HttpServlet {  
  
    private static final long serialVersionUID = 6200724895227971697L;  
  
    public void doPost(HttpServletRequest request, HttpServletResponse response)  
            throws ServletException, IOException {  
        processRequest(request, response);  
    }  
  
    // RPC transactions may be encoded as a GET request, so handle those as well  
    public void doGet(HttpServletRequest request, HttpServletResponse response)  
            throws ServletException, IOException {  
        processRequest(request, response);  
    }  

    public static Context getInitialContext() throws javax.naming.NamingException {
        Properties p = new Properties();
        p.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.NamingContextFactory");
        p.put(Context.PROVIDER_URL, "jnp://localhost:1099");
        p.put(Context.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces");
        return new javax.naming.InitialContext(p);
    }
    
    /** 
     * Servlet entry point to process the request. sends random stock updates to 
     * the "stockQuotes" channel for 30 seconds 
     *  
     * @param request The HttpServletRequest 
     * @param response The HttpServletResponse 
     * @exception ServletException As per HttpServlet.service() 
     * @exception IOException As per HttpServlet.service() 
     *  
     * @visibility external 
     */  
    public void processRequest(HttpServletRequest request,  
            HttpServletResponse response) throws ServletException, IOException {  
        new Thread() {  
            public void run() {  
                Context context = null;
                TopicSession session = null;
                TopicPublisher publisher  = null;
        		TopicConnection connection = null;
        		ObjectMessage message = null;
            	try {  
                	Random generator = new Random();
            		try {
            			context = getInitialContext();
            			TopicConnectionFactory factory = (TopicConnectionFactory)context.lookup("TopicConnectionFactory");
            			Topic topic = (Topic)context.lookup("stockQuotes");            		
            			// create the connection and session
            			connection = factory.createTopicConnection();
            			session = connection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            			// create a publisher
            			publisher = session.createPublisher(topic);
            			publisher.setDeliveryMode(DeliveryMode.NON_PERSISTENT);            			
            			// create the message
            			message = session.createObjectMessage();
            			
            		} catch (NamingException e){
            			e.printStackTrace();
            			System.out.println("ERROR NamingException");
            		} catch (JMSException e) {
            			e.printStackTrace();
            			System.out.println("ERROR JMSException");
            		}
                    List<Object[]> stockData = new ArrayList<Object[]>();  
                    long endTime = System.currentTimeMillis() + 90000;  
                    while (System.currentTimeMillis() < endTime) {  
                        stockData.clear();  
                        for (int i = 1; i <= 10; i++) {  
                            stockData.add(new Object[] { i, 0 });  
                        }  
                        int cnt = 1 + generator.nextInt(3);  
                        for (int i = 0; i < cnt; i++) {  
                            int id = 1 + generator.nextInt(10);
                            for (Object[] sd : stockData) {  
                                if (((Integer) sd[0]).intValue() == id) {  
                                    sd[1] = ((float) (generator.nextInt(101) - 50)) / 100;  
                                    break;  
                                }  
                            }  
                        }  
                        message.setObject(stockData.toArray());
                        publisher.publish(message);
                        Thread.sleep(500);  
                    }  
                } catch (Exception e) {  
                    e.printStackTrace(System.out);  
                }
                try {
                session.close();
                publisher.close();
                context.close();
                connection.close();
                } catch(Exception e) {}
            };  
        }.start();  
    }
}
